// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
// 
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file processGcode.cpp
// Main Gcode parser entry point.
// 
// @brief CNC gcode interpreter for 3 axis CNC machine. Configured for ELFSHL1
//     3 axis stepper motor driver shield.
//
// Acknowledgements:
// Inspired on the work of Carlos guilarte y diego colonnello (Txapuzas CNC).
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#include <Arduino.h>
#include "configMe.h"
#include "stepperCntrl.h"
#include "processGcode.h"

//our command string
char     inBuffer[COMMAND_SIZE];
uint16_t serial_count;

stepperCntrl *myController = NULL;

static void initBuffer (void)
{
   memset (inBuffer, 0, sizeof(inBuffer) );
   serial_count = 0;
}

//init our string processing
void initGcodeProcessor( stepperCntrl *iController )
{
   //init our command
   initBuffer ( );
   myController = iController;
}

void processGcode ( void )
{
   char c;
   static int no_data = 0;
   
   //read in characters if we got them.
   if (Serial.available() > 0)
   {
      c = Serial.read();
      no_data = 0;
      
      //newlines are ends of commands.
      if (c != '\n')
      {
         inBuffer[serial_count] = c;
         serial_count++;
      }
   }
   //mark no data.
   else
   {
      no_data++;
      delayMicroseconds(100);
   }
   
   //if theres a pause or we got a real command, do it
   if ((serial_count > 0) && (c == '\n' || no_data > 100))
   {    
      //process our command!
      process_string(inBuffer, serial_count);
      //clear command.
      initBuffer();
   }
   
   
   //no data?  turn off steppers
   if (no_data > 10000)
   {
      myController->disableSteppers();
   }
}



//Read the string and execute instructions
static void process_string(char instruction[], int size)
{
   stepperCntrl::t_floatPoint  fp;
   fp.x = 0.0;
   fp.y = 0.0;
   fp.z = 0.0;
   
   
   
   //the character / means delete block... used for comments and stuff.
   if (instruction[0] == '/')
   {
      Serial.println("ok");
      return;
   }
      
   uint8_t code = 0;
   
   //did we get a gcode?
   if (
       has_command('G', instruction, size) ||
       has_command('X', instruction, size) ||
       has_command('Y', instruction, size) ||
       has_command('Z', instruction, size)
       )
   {
      //which one?
      code = (int)search_string('G', instruction, size);
      
      // Get co-ordinates if required by the code type given
      switch (code)
      {
         case 0:
         case 1:
         case 2:
         case 3:
         case 30:
            if ( has_command('F', instruction, size) )
            {
               myController->setFeedRate(search_string('F', instruction, size));
            }

            fp = myController->getPosition();            

            // Get movement
            if(myController->absoluteCoord())
            {
               //we do it like this to save time. makes curves better.
               //eg. if only x and y are specified, we dont have to waste time looking up z.
               if (has_command('X', instruction, size))
                  fp.x = search_string('X', instruction, size);
               if (has_command('Y', instruction, size))
                  fp.y = search_string('Y', instruction, size);
               if (has_command('Z', instruction, size))
                  fp.z = search_string('Z', instruction, size);
            }
            else
            {
               fp.x += search_string('X', instruction, size);
               fp.y += search_string('Y', instruction, size);
               fp.z += search_string('Z', instruction, size);
            }
            break;
      }
      
      //do something!
      switch (code)
      {
            //Rapid Positioning
            //Linear Interpolation
            //these are basically the same thing.
         case 0:
         case 1:
            //set our target.
            myController->setTarget(fp.x, fp.y, fp.z);
            
            //finally move.
            myController->move();
            break;
            
            //Clockwise arc
         case 2:
            //Counterclockwise arc
         case 3:
            myController->moveArc (fp, search_string('I', instruction, size),
                                   search_string('J', instruction, size), 
                                   (code == 2));
            
            break;
            
            //Dwell
         case 4:
            delay((int)search_string('P', instruction, size));
            break;
            
            //Inches for Units
         case 20:
            myController->setUnits(MM_PER_INCH);
            myController->setCurveSection(CURVE_SECTION_INCHES);
            myController->computeDeltas();
            break;
            
            //mm for Units
         case 21:
            myController->setUnits(1.0);
            myController->setCurveSection(CURVE_SECTION_MM);
            myController->computeDeltas();
            break;

            //go home via an intermediate point.
         case 30:
            
            myController->setTarget(fp.x, fp.y, fp.z);
            //go there.
            myController->move();
            
            //go home.
         case 28:
            myController->setTarget(0.0, 0.0, 0.0);
            myController->move();
            break;

            //Absolute Positioning
         case 90:
            myController->setAbsoluteCoord();
            break;
            
            //Incremental Positioning
         case 91:
            myController->setRelativeCoord();
            break;
            
            //Set as home
         case 92:
            myController->setPosition(0.0, 0.0, 0.0);
            break;
            
            /*
             //Inverse Time Feed Mode
             case 93:
             
             break;  //TODO: add this
             
             //Feed per Minute Mode
             case 94:
             
             break;  //TODO: add this
             */
            
         default:
            Serial.print("error ");
            Serial.println(code,DEC);
      }
   }
   
   //find us an m code.
   if (has_command('M', instruction, size))
   {
      code = search_string('M', instruction, size);
      switch (code)
      {
         case 0:
            break;
         case 3:
            //motor(HIGH);
            Serial.println("CNC:Motor ON");
            //digitalWrite(MOTOR_PIN,HIGH);
            break;
         case 5:
            //motor(LOW);
            Serial.println("CNC:Motor OFF");
            break;	
         case 105:
            Serial.print("T:");
            Serial.println (0.0,2);
            break;
         case 114:
            Serial.println("C:0.0 0.0");
            break;
         default:
            Serial.print("error ");
            Serial.println(code, DEC);
      }		
   }
   
   //tell our host we're done.
   Serial.println("ok");
   //	Serial.println(line, DEC);
}

//look for the number that appears after the char key and return it
static double search_string(char key, char instruction[], int string_size)
{
   char temp[10] = "";
   
   for (byte i=0; i<string_size; i++)
   {
      if (instruction[i] == key)
      {
         i++;      
         int k = 0;
         while (i < string_size && k < 10)
         {
            if (instruction[i] == 0 || instruction[i] == ' ')
               break;
            
            temp[k] = instruction[i];
            i++;
            k++;
         }
         return strtod(temp, NULL);
      }
   }
   
   return 0;
}

//look for the command if it exists.
static bool has_command(char key, char instruction[], int string_size)
{
   for (byte i=0; i<string_size; i++)
   {
      if (instruction[i] == key)
         return true;
   }
   
   return false;
}

