// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
// 
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file stepperCntrl.cpp
// Main machine controller. 
// 
// @brief This class does all the coordinated movement of the stepper motors in
// terms of moving the three axis to a given coordinate. This class controls
// the linear movements, controls the machine's coordinates and positioning, 
// axis velocity.
//
// Acknowledgements:
// Inspired on the work of Carlos guilarte y diego colonnello (Txapuzas CNC).
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#ifndef _stepperCntrl_h_
#define _stepperCntrl_h_

#include "configMe.h"
#include "stepper.h"

class stepperCntrl
{
   // our point structure to make things nice.
   typedef struct LongPoint 
   {
      long x;
      long y;
      long z;
   } t_longPoint;

   
   typedef struct u8Point
   {
      uint8_t x;
      uint8_t y;
      uint8_t z;
   } t_u8Point;
   
public:
   
   typedef struct FloatPoint 
   {
      float x;
      float y;
      float z;
   } t_floatPoint;
   
   stepperCntrl ( void );
   void init (void );
   
   // Coordinate and target locations
   void setTarget(float x, float y, float z);
   void setPosition(float x, float y, float z);
   t_floatPoint getPosition ( void );
   void  computeDeltas (void);
   void  setUnits (float units);
   void  setCurveSection ( float curveSection );
   float getCurveSection ( );
   void  setAbsoluteCoord ( );
   void  setRelativeCoord ( );
   bool  absoluteCoord ( void );
   
   
   // Feed rate and movement velocity methods
   long  getMaxSpeed();   
   void  setFeedRate ( long feedDelay );

   // Machine movement methods
   void  move ( void );
   void  moveArc ( t_floatPoint fp, double i, double j, bool cwMove );

   // Stepper driver control methods.
   void  disableSteppers ();
   void  enableSteppers ();


private:
   // Convertion methods, positioning and movement methods
   long  toSteps ( float stepsPerUnit, float units );

   
   void setDirection ( uint8_t dirX, uint8_t dirY, uint8_t dirZ );
   bool canStep (uint8_t min_pin, uint8_t max_pin, long current, long target, 
                 uint8_t direction);

   // Read end-stops
   bool  readSwitch(uint8_t pin);
   
   // Machine time and feedrate methods
   long  calculateFeedrateDelay(long feedrate);
   void  setFeedDelay ( );

   
   //our feedrate variables.
   long  _feedRate;              // Feed rate expressed in mm/min
   long  _feedrateDelay;         // Feed rate delay - time take to move to target base on feedrate
   long  _lastSpeed;             // Last speed set

   
   float       _units_in_mm;     // Units expressed in mm. 1 inch 25.4 mm      
   t_longPoint _stepsPerUnit;    // Number of steps per configured units
   t_u8Point   _direction;       // Movement direction for each axis - 0 or 1
   
   t_floatPoint _current_units;  // Current position expressed in current units (mm or inch)
   t_floatPoint _target_units;   // Target position expressed in current units (mm or inch)
   t_floatPoint _delta_units;    // Difference between current position and target position
   
   t_longPoint _current_steps;   // Current position expressed in steps
   t_longPoint _target_steps;    // Target position expressed in steps
   t_longPoint _delta_steps;     // Difference between current and target position
   
   float _curveSection;          // Curve section
   
   stepper *_xStepper;           // Stepper driver controlling X axis.
   stepper *_yStepper;           // Stepper driver controlling Y axis. 
   stepper *_zStepper;           // Stepper driver controlling Z axis.
   
   bool _absMode;
  
};

#endif